#version 100
precision highp float;
precision highp int;

varying vec4 v_color;
varying vec2 v_texCoords;

uniform sampler2D u_texture;
uniform vec2 uv_offset;
uniform vec2 uv_repeat;
uniform float time;

vec4 back_color = vec4(1,1,1,0);
vec4 front_color = vec4(1,1,1,1);


float Hash(vec2 p)
{
    float h = dot(p, vec2(127.1, 311.7));
    return -1.0 + 2.0 * fract(sin(h) * 4358.5453);
}


float Noise(vec2 p)
{
    vec2 i = floor(p);
    vec2 f = fract(p);
    vec2 u = f * f * (3.0 - 2.0 * f);

    return mix(mix(Hash(i + vec2(0.0, 0.0)),
                   Hash(i + vec2(1.0, 0.0)), u.x),
               mix(Hash(i + vec2(0.0, 1.0)),
                   Hash(i + vec2(1.0, 1.0)), u.x), u.y);

}

float Voronoi(vec2 x)
{
    ivec2 p = ivec2(floor(x));
    vec2  f = fract(x);

    float res = 8.0;
    for(int j = -1; j <= 1; ++j)
        for(int i = -1; i <= 1; ++i) {
        ivec2 b = ivec2(i, j);
        vec2  r = vec2(b) - f + 0.5 + 0.4 * sin((time * 10.0) + 6.2831 * Noise(vec2(p + b)));
        float d = dot(r, r);
        res = min(res, d);
    }
    return sqrt(res);
}

vec3 VoronoiFn(vec2 x)
{
    ivec2 p = ivec2(floor(x));
    vec2 f = fract(x);

    float f1 = 18.0;
    float f2 = 18.0;
    float f3 = 18.0;
    for (int j = -1; j <= 1; ++ j) {
        for (int i = -1; i <= 1; ++ i) {
            ivec2 b = ivec2(i, j);
            vec2 r = vec2(b) + 0.5 + 0.4 * sin((time * 10.0) + 6.2831 * Hash(vec2(p + b))) - f;;
            float d = dot(r, r);

            if (d < f1) {
                f3 = f2;
                f2 = f1;
                f1 = d;
            } else if (d < f2) {
                f3 = f2;
                f2 = d;
            } else if (d < f3) {
                f3 = d;
            }
        }
    }
    return vec3(f1, f2, f3);
}
float oscillate(float vMin, float vMax, float t) {
    float halfRange = (vMax - vMin) / vMax * 0.5;
    return (sin(t) * halfRange + (1.0 - halfRange)) * vMax;
}

float makeDisc(vec2 p, float b, float r) {
	return smoothstep(r + b, r - b, sqrt(dot(p, p)));
}
vec2 rotateUV(vec2 uv, float rotation, vec2 mid)
{
    return vec2(
      cos(rotation) * (uv.x - mid.x) + sin(rotation) * (uv.y - mid.y) + mid.x,
      cos(rotation) * (uv.y - mid.y) - sin(rotation) * (uv.x - mid.x) + mid.y
    );
}

void main()
{

    vec2 toxicuv = v_texCoords * vec2(0.25, 0.25) + vec2(0.0, 0.75) ;
    toxicuv = rotateUV(toxicuv, time * 1.24  , vec2(0.0 + 0.125, 0.75 + 0.125));
    vec4 toxic = texture2D(u_texture, toxicuv );


    vec2 maskuv = v_texCoords * vec2(0.25, 0.25) + vec2(0.75, 0.75) ;
    vec4 mask = texture2D(u_texture, maskuv + ((toxic.a )) * 0.008 );

    vec2 disc_center = vec2(0.5, 0.5);
    vec2 diskpos = v_texCoords;
    diskpos -= disc_center;
    vec4 glow1disk = mix(vec4(0,0,0,0), vec4(1,0,0,1), makeDisc(diskpos, 0.2, 0.35));// * oscillate(0.0, 1.0, alpha * 12.5);

    float x = (disc_center.x - v_texCoords.x);
    float y = (disc_center.y - v_texCoords.y);
    float r = -(x*x + y*y) * 0.82;
    float z = 1.0 + 0.5*sin((r + time*0.1 )/0.026);
    z = pow(z, 2.5);

    vec3 f_vec = VoronoiFn(8.0 * v_texCoords.xy );

    //vec4(vec3(1,.3f,.0f), toxic.a );// + vec4(v_color.rgb, v_color.a) *  clamp(vec4(1.0,1.0,1.0, mask.a  * 0.6), vec4(0), vec4(1)) ;
    //gl_FragColor = mix(vec4(0,0,0,0), vec4(vec3(1,.3f,.0f), mask.r*oscillate(0.1, 0.79, time * 0.98f)), glow1disk.a );//vec4(vec3(1,.3f,.0f), toxic.a );// + vec4(v_color.rgb, v_color.a) *  clamp(vec4(1.0,1.0,1.0, mask.a  * 0.6), vec4(0), vec4(1)) ;
     gl_FragColor = vec4(vec3(0.7,0.4,0.2), mask.r * oscillate(0.1, 0.76, time * 0.98)) ;
}
